////////////////////////////////////////////////////////////
// X88000 Resource
//
// Written by Manuke

////////////////////////////////////////////////////////////
// include

#include "StdHeader.h"

#include "X88Resource.h"

#include "X88Utility.h"

using namespace NX88Utility;

////////////////////////////////////////////////////////////
// implementation of NX88Resource

////////////////////////////////////////////////////////////
// static

// UI language

static int s_nUILanguage = NX88Resource::X88LANG_AUTO;

// locale language

static int s_nLocaleLanguage = NX88Resource::X88LANG_AUTO;

// resource string map initialized

static bool s_bResStrMapInitialized = false;

////////////////////////////////////////////////////////////
// language

// get UI language

int NX88Resource::GetUILanguage() {
	return s_nUILanguage;
}

// set UI language

void NX88Resource::SetUILanguage(int nUILanguage) {
	if (nUILanguage != s_nUILanguage) {
		s_nUILanguage = nUILanguage;
		s_bResStrMapInitialized = false;
	}
}

// get locale language

int NX88Resource::GetLocaleLanguage() {
	if (s_nLocaleLanguage == X88LANG_AUTO) {
		s_nLocaleLanguage = X88LANG_ENGLISH;

#ifdef X88_PLATFORM_WINDOWS

		LCID lcid = GetThreadLocale();
		if (PRIMARYLANGID(LANGIDFROMLCID(lcid)) == LANG_JAPANESE) {
			s_nLocaleLanguage = X88LANG_JAPANESE;
		}

#elif defined(X88_PLATFORM_UNIX)

		std::string strLocale;
		const char* pszLocale = setlocale(LC_MESSAGES, NULL);
		if (pszLocale != NULL) {
			strLocale = pszLocale;
			StrToLwr(strLocale);
			while ((strLocale.size() > 0) && isspace(strLocale[0])) {
				strLocale.erase(0, 1);
			}
		}
		if ((strLocale.find("ja") == 0) ||
			(strLocale.find("ja_jp") != std::string::npos) ||
			(strLocale.find("japan") != std::string::npos) ||
			(strLocale.find("euc-jp") != std::string::npos) ||
			(strLocale.find("euc_jp") != std::string::npos) ||
			(strLocale.find("eucjp") != std::string::npos))
		{
			s_nLocaleLanguage = X88LANG_JAPANESE;
		}

#endif // X88_PLATFORM

	}
	return s_nLocaleLanguage;
}

// get resource language

int NX88Resource::GetResLanguage() {
	int nResLanguage = GetUILanguage();
	if (nResLanguage == X88LANG_AUTO) {
		nResLanguage = GetLocaleLanguage();
	}
	return nResLanguage;
}

// is resource language japanese mode

bool NX88Resource::IsResJapanese() {
	return GetResLanguage() == X88LANG_JAPANESE;
}

////////////////////////////////////////////////////////////
// string resource

// get version(GUI encoding)

std::string NX88Resource::GetVersion() {
	return "1.4.0";
}

// get resource string(GUI encoding)

const char* NX88Resource::GetResStr(const char* psszOrg) {
	static const char* apsszResStr[][2] = {
		// image file
		"<none>",
			"<Ȃ>",
		// parallel device
		"NULL",
			"Ȃ",
		// printer paper
		"10 inch continuous",
			"10C`A",
		"15 inch continuous",
			"15C`A",
		"A5 Portrait",
			"A5c",
		"A5 Landscape",
			"A5",
		"A4 Portrait",
			"A4c",
		"A4 Landscape",
			"A4",
		"A3 Portrait",
			"A3c",
		"A3 Landscape",
			"A3",
		"B5 Portrait",
			"B5c",
		"B5 Landscape",
			"B5",
		"B4 Portrait",
			"B4c",
		"B4 Landscape",
			"B4",
		"Postcard Portrait",
			"͂c",
		"Postcard Landscape",
			"͂",
		// message caption
		"Breakpoint : Address",
			"u[N|Cg:AhX",
		"Disassemble : Address",
			"tAZu:AhX",
		"Disassemble : Line Count",
			"tAZu:s",
		"Dump : Address",
			"_v:AhX",
		"Dump : Byte Count",
			"_v:oCg",
		"Disk Image",
			"fBXNC[W",
		"Export Screenshot",
			"摜o",
		"Select Paper",
			"pI",
		"Printing Image",
			"C[W",
		"Delete Paper",
			"p폜",
		"Printer",
			"v^",
		"Tape Image",
			"e[vC[W",
		"Saved Data",
			"Z[uf[^",
		// message text
		"Please enter a number between %d-%d",
			"%dȏ%dȉ̐l͂Ă",
		"Please enter a hexadecimal number between %04XH-%04XH",
			"%04XHȏ%04XHȉ16i͂Ă",
		"This address is already registered.",
			"̃AhX͓o^ς݂ł",
		"An error occurred while reading.",
			"ǂݍ݃G[",
		"An error occurred while writing.",
			"oG[",
		"Delete paper?",
			"p폜Ă낵łH",
		"Delete page %d?",
			"y[W %d 폜Ă낵łH",
		"Delete all pages?",
			"Sy[W폜Ă낵łH",
		"Reset Printer?",
			"v^Zbg܂B낵łH",
		"Erase saved data?",
			"Z[uf[^Ă낵łH",
		// file dialog caption
		"Add Disk Image",
			"fBXNC[W̒ǉ",
		"Open Tape Image",
			"e[vC[WJ",
		"Save Tape Image",
			"e[vC[Wۑ",
		"Load Memory Image",
			"C[Wǂݍ",
		"Export Screenshot(XWD)",
			"摜o(XWD)",
		"Write PostScript",
			"PostScripto",
		"Write PDF",
			"PDFo",
		"Write Text",
			"eLXgo",
		// file dialog filter
		"Disk Image(d88)|*.d88||",
			"fBXNC[W(d88)|*.d88||",
		"Tape Image(t88, cmt)|*.t88;*.cmt||",
			"e[vC[W(t88, cmt)|*.t88;*.cmt||",
		"Memory Image(n80)|*.n80||",
			"C[W(n80)|*.n80||",
		"Screenshot(XWD)|*.xwd||",
			"摜(XWD)|*.xwd||",
		"PostScript(ps)|*.ps||",
			"PostScript(ps)|*.ps||",
		"PDF(pdf)|*.pdf||",
			"pdf(pdf)|*.pdf||",
		"Text(txt)|*.txt||",
			"eLXg(txt)|*.txt||",
		// other dialog item
		"Environment Settings",
			"ݒ",
		"General",
			"",
		"Display",
			"",
		"Input",
			"",
		"Sound",
			"TEh",
		"Auto",
			"",
		"Japanese",
			"{",
		"English",
			"p",
		"20 FPS",
			"20t[/b",
		"30 FPS",
			"30t[/b",
		"40 FPS",
			"40t[/b",
		"60 FPS",
			"60t[/b",
		"8 bit",
			"8rbg",
		"16 bit",
			"16rbg",
		"Mono",
			"m",
		"Stereo",
			"XeI",
		"Same",
			"{",

#ifdef X88_GUI_WINDOWS

		// display font
		"Courier New",
			"lr SVbN",
		// printer font
		"MS Mincho",
			"lr ",
		"@MS Mincho",
			"@lr ",
		"MS Gothic",
			"lr SVbN",
		"@MS Gothic",
			"@lr SVbN",

#elif defined(X88_GUI_GTK)

		// printer font
		"MS Mincho",
			"lr ",
		"MS Gothic",
			"lr SVbN",
		// dialog template
		"UI Language",
			"UI",
		"Boost Limiter",
			"u[Xg~b^",
		"Disk Drives",
			"fBXN",
		"1 Drive",
			"1hCu",
		"2 Drives",
			"2hCu",
		"3 Drives",
			"3hCu",
		"4 Drives",
			"4hCu",
		"Other",
			"̑",
		"Wait",
			"EFCg",
		"Older Compatibility",
			"@݊",
		"International Mode",
			"COŃ[h",
		"Frame Rate",
			"XV[g",
		"Interlace",
			"C^[X",
		"ON",
			"",
		"OFF",
			"Ȃ",
		"Use",
			"gp",
		"Don't Use",
			"gpȂ",
		"Enable",
			"",
		"Disable",
			"֎~",
		"Printer Settings",
			"v^ݒ",
		"Printer Name",
			"v^",
		"Paper Size",
			"pTCY",
		"Centering",
			"p",
		"Page Number",
			"y[W",
		"Write",
			"o",
		"Delete Paper",
			"p폜",
		"Feed Paper",
			"",
		"Reset",
			"Zbg",
		"Preview",
			"vr[",
		"Printer Settings : Write",
			"v^ݒ:o",
		"Content",
			"e",
		"Zoom Rate",
			"䗦",
		"Border",
			"g",
		"Sprocket Hole",
			"XvPbg",
		"Text Extraction",
			"eLXgo",
		"Printer Settings : Delete Paper",
			"v^ݒ:p폜",
		"Target",
			"Ώ",
		"Specified Page",
			"wy[W",
		"All Pages",
			"Sy[W",
		"Printer Settings : Feed Paper",
			"v^ݒ:",
		"Unit",
			"P",
		"Page",
			"y[W",
		"Line",
			"s",
		"Point",
			"|Cg",
		"Disk Image",
			"fBXNC[W",
		"Disk Image File",
			"fBXNC[Wt@C",
		"File",
			"t@C",
		"Add",
			"ǉ",
		"Remove",
			"폜",
		"Remove All",
			"S폜",
		"Disk Drive",
			"hCu",
		"Drive 1",
			"hCu1",
		"Drive 2",
			"hCu2",
		"Drive 3",
			"hCu3",
		"Drive 4",
			"hCu4",
		"Tape Image",
			"e[vC[W",
		"Image for Loading",
			"[hC[W",
		"Open",
			"J",
		"Image for Saving",
			"Z[uC[W",
		"Save",
			"ۑ",
		"Erase",
			"",
		"Export RAM",
			"RAMo",
		"Main RAM0 : 0000H-7FFFH",
			"CRAM0 : 0000H-7FFFH",
		"Main RAM1 : 8000H-FFFFH",
			"CRAM1 : 8000H-FFFFH",
		"Fast Text VRAM",
			"eLXgVRAM",
		"Slow Text VRAM",
			"ᑬeLXgVRAM",
		"Graphic VRAM0",
			"OtBbNVRAM0",
		"Graphic VRAM1",
			"OtBbNVRAM1",
		"Graphic VRAM2",
			"OtBbNVRAM2",
		"Subsystem RAM",
			"TuVXeRAM",
		"Expansion RAM0",
			"gRAM0",
		"Expansion RAM1[VAB]",
			"gRAM1[VAB]",
		"Disassemble",
			"tAZu",
		"Range",
			"͈",
		"Address",
			"AhX",
		"Lines",
			"s",
		"Refresh",
			"ĕ\",
		"Disassembled List",
			"tAZuXg",
		"Dump",
			"_v",
		"Byte Count",
			"oCg",
		"Dump List",
			"_vXg",
		"Breakpoint Settings",
			"u[N|Cgݒ",
		"Position",
			"ʒu",
		"Breakpoint",
			"u[N|Cg",
		"Version Information",
			"o[W",

		// menu
		"/_System",
			"/VXe(_S)",
		"/System/_Reset",
			"/VXe(S)/Zbg(_R)",
		"/System/---",
			"/VXe(S)/---",
		"/System/B_ASIC Mode",
			"/VXe(S)/BASIC[h(_A)",
		"/System/BASIC Mode/_N-BASIC Mode",
			"/VXe(S)/BASIC[h(A)/N-BASIC[h(_N)",
		"/System/BASIC Mode/N88-BASIC V_1-S Mode",
			"/VXe(S)/BASIC[h(A)/N88-BASIC V1-S[h(_1)",
		"/System/BASIC Mode/N88-BASIC V1-_H Mode",
			"/VXe(S)/BASIC[h(A)/N88-BASIC V1-H[h(_H)",
		"/System/BASIC Mode/N88-BASIC V_2 Mode",
			"/VXe(S)/BASIC[h(A)/N88-BASIC V2[h(_2)",
		"/System/BASIC Mode/---",
			"/VXe(S)/BASIC[h(A)/---",
		"/System/BASIC Mode/N_80-BASIC V1 Mode",
			"/VXe(S)/BASIC[h(A)/N80-BASIC V1[h(_8)",
		"/System/BASIC Mode/N8_0-BASIC V2 Mode",
			"/VXe(S)/BASIC[h(A)/N80-BASIC V2[h(_0)",
		"/System/_Clock",
			"/VXe(S)/NbN(_C)",
		"/System/Clock/_4MHz Mode",
			"/VXe(S)/NbN(C)/4MHz[h(_4)",
		"/System/Clock/_8MHz Mode",
			"/VXe(S)/NbN(C)/8MHz[h(_8)",
		"/System/_Boost Mode",
			"/VXe(S)/u[Xg[h(_B)",
		"/System/_Fullscreen Mode",
			"/VXe(S)/tXN[[h(_F)",
		"/System/---",
			"/VXe(S)/---",
		"/System/C_lipboard",
			"/VXe(S)/Nbv{[h(_L)",
		"/System/Clipboard/Export Screenshot : XWD(_C)",
			"/VXe(S)/Nbv{[h(L)/摜oFXWD(_C)",
		"/System/Clipboard/Copy : _Text",
			"/VXe(S)/Nbv{[h(L)/Rs[FeLXg(_T)",
		"/System/Clipboard/_Paste : Text",
			"/VXe(S)/Nbv{[h(L)/tFeLXg(_P)",
		"/System/---",
			"/VXe(S)/---",
		"/System/_Environment Settings...",
			"/VXe(S)/ݒ(_E)...",
		"/System/_Parallel Port",
			"/VXe(S)/p|[g(_P)",
		"/System/Parallel Port/_Device Settings...",
			"/VXe(S)/p|[g(P)/foCX̐ݒ(_D)...",
		"/System/Parallel Port/---",
			"/VXe(S)/p|[g(P)/---",
		"/System/Parallel Port/%s(_0)",
			"/VXe(S)/p|[g(P)/%s(_0)",
		"/System/Parallel Port/%s(_1)",
			"/VXe(S)/p|[g(P)/%s(_1)",
		"/System/Parallel Port/%s(_2)",
			"/VXe(S)/p|[g(P)/%s(_2)",
		"/System/Parallel Port/%s(_3)",
			"/VXe(S)/p|[g(P)/%s(_3)",
		"/System/Parallel Port/%s(_4)",
			"/VXe(S)/p|[g(P)/%s(_4)",
		"/System/Parallel Port/%s(_5)",
			"/VXe(S)/p|[g(P)/%s(_5)",
		"/System/Parallel Port/%s(_6)",
			"/VXe(S)/p|[g(P)/%s(_6)",
		"/System/Parallel Port/%s(_7)",
			"/VXe(S)/p|[g(P)/%s(_7)",
		"/System/Parallel Port/%s(_8)",
			"/VXe(S)/p|[g(P)/%s(_8)",
		"/System/Parallel Port/%s(_9)",
			"/VXe(S)/p|[g(P)/%s(_9)",
		"/System/---",
			"/VXe(S)/---",
		"/System/E_xit X88000",
			"/VXe(S)/X88000̏I(_X)",
		"/_Image",
			"/C[W(_I)",
		"/Image/_Disk Image...",
			"/C[W(I)/fBXNC[W(_D)...",
		"/Image/_Tape Image...",
			"/C[W(I)/e[vC[W(_T)...",
		"/Image/_Memory Image...",
			"/C[W(I)/C[W(_M)...",
		"/Debu_g",
			"/fobO(_G)",
		"/Debug/Main CPU Debug(_1)",
			"/fobO(G)/CCPUfobO(_1)",
		"/Debug/Sub CPU Debug(_2)",
			"/fobO(G)/TuCPUfobO(_2)",
		"/Debug/---",
			"/fobO(G)/---",
		"/Debug/Execute Debu_g",
			"/fobO(G)/fobOs(_G)",
		"/Debug/---",
			"/fobO(G)/---",
		"/Debug/Execute _Step",
			"/fobO(G)/Xebvs(_S)",
		"/Debug/Execute _Trace",
			"/fobO(G)/g[Xs(_T)",
		"/Debug/Execute Ste_p2",
			"/fobO(G)/Xebvs2(_P)",
		"/Debug/---",
			"/fobO(G)/---",
		"/Debug/Record Execution L_og",
			"/fobO(G)/sOL^(_O)",
		"/Debug/Export RAM(_W)...",
			"/fobO(G)/RAMo(_W)...",
		"/Debug/---",
			"/fobO(G)/---",
		"/Debug/Disassemb_le...",
			"/fobO(G)/tAZu(_L)...",
		"/Debug/_Dump...",
			"/fobO(G)/_v(_D)...",
		"/Debug/_Breakpoint Settings...",
			"/fobO(G)/u[N|Cgݒ(_B)...",
		"/_Help",
			"/wv(_H)",
		"/Help/_About...",
			"/wv(H)/o[W(_A)...",

#endif // X88_GUI

		NULL, NULL
	};
	static std::map<std::string, std::string> mapResStr;
	if (!s_bResStrMapInitialized) {
		s_bResStrMapInitialized = true;
		mapResStr.clear();
		if (IsResJapanese()) {
			for (int i = 0; apsszResStr[i][0] != NULL; i++) {
				mapResStr[apsszResStr[i][0]] =
					ConvSRCtoGUI(apsszResStr[i][1]);
			}
		}
	}
	const char* pgszRes = psszOrg;
	if (psszOrg != NULL) {
		std::map<std::string, std::string>::iterator it =
			mapResStr.find(psszOrg);
		if (it != mapResStr.end()) {
			pgszRes = (*it).second.c_str();
		}
	}
	return pgszRes;
}

// get formatted resource string(GUI encoding)

std::string NX88Resource::FormatResStr(const char* psszOrg, ...) {
	std::string gstrRes;
	const char* pgszOrg = GetResStr(psszOrg);
	if (pgszOrg != NULL) {
		va_list ap;
		va_start(ap, psszOrg);
		gstrRes = FormatStrV(pgszOrg, ap);
		va_end(ap);
	}
	return gstrRes;
}

// get parallel device name from id(GUI encoding)

std::string NX88Resource::GetParallelDeviceNameFromID(
	const std::string& strDeviceID)
{
	static const char* apszDeviceNames[][2] = {
		"null", "NULL",
		"pr201", "PC-PR201"
	};
	static const int nDeviceCount =
		sizeof(apszDeviceNames)/sizeof(apszDeviceNames[0]);
	std::string gstrDeviceName;
	for (int i = 0; i < nDeviceCount; i++) {
		if (StrCaseCmp(strDeviceID, apszDeviceNames[i][0]) == 0) {
			gstrDeviceName = RSTR(apszDeviceNames[i][1]);
			break;
		}
	}
	return gstrDeviceName;
}

////////////////////////////////////////////////////////////
// dialog resource

#ifdef X88_GUI_GTK

// get dialog entries

NX88Resource::SDialogEntry*
	NX88Resource::GetDialogEntries(int nID, int& nItemCount)
{
	// message-box
	static SDialogEntry adeMessageBox[] = {
		{	// ActionArea
			" Button", "OK",
			1, IDOK, NULL, "fd"
		}, {
			" Button", "Yes",
			4, IDYES, NULL, "f"
		}, {
			" Button", "No",
			8, IDNO, NULL, "f"
		}, {
			" Button", "Cancel",
			2, IDCANCEL, NULL, "f"
		}, {	// MainVBox
			"VBox", NULL,
			0, 0, "b1", NULL
		}, {
			" Label", NULL,
			0, 0, "s50,1", NULL
		}, {	// Message
			" Label", "",
			0, IDC_MESSAGE, NULL, NULL
		}, {
			" Label", NULL,
			0, 0, "s50,1", NULL
		}
	};
	// environment settings dialog
	static SDialogEntry adeEnvSet[] = {
		{
			"Caption", "Environment Settings",
			0, 0, NULL, NULL
		}, {	// ActionArea
			" Button", "OK",
			0, IDOK, NULL, "fd"
		}, {
			" Button", "Cancel",
			0, IDCANCEL, NULL, "f"
		}, {	// MainVBox
			"+*NoteBook", NULL,
			0, 0, "b1", NULL
		}, {	// General Page
			"NoteTag", "General",
			0, 0, NULL, NULL
		}, {
			" HBox", NULL,
			0, IDC_ENV_GENERAL, "b1", NULL
		}, {	// Display Page
			"NoteTag", "Display",
			0, 0, NULL, NULL
		}, {
			" HBox", NULL,
			0, IDC_ENV_DISPLAY, "b1", NULL
		}, {	// Input Page
			"NoteTag", "Input",
			0, 0, NULL, NULL
		}, {
			" HBox", NULL,
			0, IDC_ENV_INPUT, "b1", NULL
		}
	};
	// environment settings: general notebook
	static SDialogEntry adeEnvGeneral[] = {
		{
			"HBox", NULL,
			0, 0, NULL, NULL
		}, {
			" VBox", NULL,
			0, 0, NULL, NULL
		}, {
			"  HBox", NULL,
			0, 0, NULL, NULL
		}, {
			"   Label", "UI Language",
			0, 0, NULL, NULL
		}, {
			"  Combo", NULL,
			0, IDC_UI_LANGUAGE, NULL, "e0"
		}, {
			"  HBox", NULL,
			0, 0, NULL, NULL
		}, {
			"   Label", "Boost Limiter",
			0, 0, NULL, NULL
		}, {
			"  Combo", NULL,
			0, IDC_BOOST_LIMITER, NULL, "e0"
		}, {
			"  Frame", "Disk Drives",
			0, 0, NULL, NULL
		}, {
			"   VBox", NULL,
			0, 0, "b1", NULL
		}, {
			"    RadioButton", "1 Drive",
			0, IDC_DISK_DRIVE1, NULL, NULL
		}, {
			"    RadioButton", "2 Drives",
			0, IDC_DISK_DRIVE2, NULL, NULL
		}, {
			"    RadioButton", "3 Drives",
			0, IDC_DISK_DRIVE3, NULL, NULL
		}, {
			"    RadioButton", "4 Drives",
			0, IDC_DISK_DRIVE4, NULL, NULL
		}, {
			" VBox", NULL,
			0, 0, NULL, NULL
		}, {
			"  Frame", "CRT",
			0, 0, NULL, NULL
		}, {
			"   VBox", NULL,
			0, 0, "b1", NULL
		}, {
			"    RadioButton", "24KHz",
			0, IDC_VSYNC_24KHZ, NULL, NULL
		}, {
			"    RadioButton", "15KHz",
			0, IDC_VSYNC_15KHZ, NULL, NULL
		}, {
			"  Frame", "Other",
			0, 0, NULL, NULL
		}, {
			"   VBox", NULL,
			0, 0, "b1", NULL
		}, {
			"    CheckButton", "Wait",
			0, IDC_WAIT_EMULATION, NULL, NULL
		}, {
			"    CheckButton", "Older Compatibility",
			0, IDC_OLD_COMPATIBLE, NULL, NULL
		}, {
			"    CheckButton", "PCG-8800",
			0, IDC_PCG_ENABLE, NULL, NULL
		}, {
			"    CheckButton", "International Mode",
			0, IDC_OPTION_FONT, NULL, NULL
		}
	};
	// environment settings: display notebook
	static SDialogEntry adeEnvDisplay[] = {
		{
			"HBox", NULL,
			0, 0, NULL, NULL
		}, {
			" VBox", NULL,
			0, 0, NULL, NULL
		}, {
			"  HBox", NULL,
			0, 0, NULL, NULL
		}, {
			"   Label", "Frame Rate",
			0, 0, NULL, NULL
		}, {
			"  Combo", NULL,
			0, IDC_FRAME_RATE, NULL, "e0"
		}, {
			"  Frame", "Interlace",
			0, 0, NULL, NULL
		}, {
			"   VBox", NULL,
			0, 0, "b1", NULL
		}, {
			"    RadioButton", "ON",
			0, IDC_INTERLACE_DO, NULL, NULL
		}, {
			"    RadioButton", "OFF",
			0, IDC_INTERLACE_NODO, NULL, NULL
		}
	};
	// environment settings: input notebook
	static SDialogEntry adeEnvInput[] = {
		{
			"HBox", NULL,
			0, 0, NULL, NULL
		}, {
			" VBox", NULL,
			0, 0, NULL, NULL
		}, {
			"  Frame", "XQueryKeymap",
			0, 0, NULL, NULL
		}, {
			"   VBox", NULL,
			0, 0, "b1", NULL
		}, {
			"    RadioButton", "Use",
			0, IDC_DIRECT_INPUT_USE, NULL, NULL
		}, {
			"    RadioButton", "Don't Use",
			0, IDC_DIRECT_INPUT_NOUSE, NULL, NULL
		}, {
			"  Frame", "IM",
			0, 0, NULL, NULL
		}, {
			"   VBox", NULL,
			0, 0, "b1", NULL
		}, {
			"    RadioButton", "Enable",
			0, IDC_IME_ENABLE, NULL, NULL
		}, {
			"    RadioButton", "Disable",
			0, IDC_IME_DISABLE, NULL, NULL
		}
	};
	// printer settings dialog
	static SDialogEntry adePrinter[] = {
		{
			"Caption", "Printer Settings",
			0, 0, NULL, NULL
		}, {	// ActionArea
			" Button", "OK",
			0, IDOK, NULL, "fd"
		}, {	// MainHBox
			"+*HBox", NULL,
			0, 0, "b1", NULL
		}, {
			" VBox", NULL,
			0, 0, "b1", NULL
		}, {
			"  HBox", NULL,
			0, 0, NULL, NULL
		}, {
			"   Label", "Printer Name",
			0, 0, NULL, NULL
		}, {
			"  HBox", NULL,
			0, 0, NULL, NULL
		}, {
			"   Label", "",
			0, IDC_PRINTER_NAME, NULL, NULL
		}, {
			"  HBox", NULL,
			0, 0, NULL, NULL
		}, {
			"   Label", "Paper Size",
			0, 0, NULL, NULL
		}, {
			"  Combo", NULL,
			0, IDC_PRINTER_PAPER, NULL, "e0"
		}, {
			"  HBox", NULL,
			0, 0, NULL, NULL
		}, {
			"   CheckButton", "Centering",
			0, IDC_PRINTER_PAPER_CENTER, NULL, NULL
		}, {
			"  HBox", NULL,
			0, 0, NULL, NULL
		}, {
			"   Label", "Page Number",
			0, 0, NULL, NULL
		}, {
			"  HBox", NULL,
			0, 0, NULL, NULL
		}, {
			"   +*SpinButton", NULL,
			0, IDC_PRINTER_PAGE_SPIN, NULL, NULL
		}, {
			"   +*HBox", NULL,
			0, 0, NULL, NULL
		}, {
			"    Label", "/1",
			0, IDC_PRINTER_PAGE_MAX, NULL, NULL
		}, {
			"  HBox", NULL,
			0, 0, NULL, NULL
		}, {
			"   VBox", NULL,
			0, 0, NULL, NULL
		}, {
			"    Button", "Write",
			0, IDC_PRINTER_COPY, NULL, NULL
		}, {
			"    Button", "Delete Paper",
			0, IDC_PRINTER_PAPER_DELETE, NULL, NULL
		}, {
			"    Button", "Feed Paper",
			0, IDC_PRINTER_PAPER_FEED, NULL, NULL
		}, {
			"    Button", "Reset",
			0, IDC_PRINTER_RESET, NULL, NULL
		}, {
			" +*VBox", NULL,
			0, 0, "b1", NULL
		}, {
			"  HBox", NULL,
			0, 0, NULL, NULL
		}, {
			"   Label", "Preview",
			0, 0, NULL, NULL
		}, {
			"  +*ScrolledWindow", NULL,
			0, IDC_PRINTER_PREVIEW, "s64,64", NULL
		}, {
			"   Alignment", NULL,
			0, IDC_PRINTER_PREVIEW_ALIGN, NULL, NULL
		}
	};
	// printer settings: write dialog
	static SDialogEntry adePrinterCopy[] = {
		{
			"Caption", "Printer Settings : Write",
			0, 0, NULL, NULL
		}, {	// ActionArea
			" Button", "OK",
			0, IDOK, NULL, "fd"
		}, {
			" Button", "Cancel",
			0, IDCANCEL, NULL, "f"
		}, {	// MainVBox
			"+*VBox", NULL,
			0, 0, "b1", NULL
		}, {
			" Frame", "Page Number",
			0, 0, NULL, NULL
		}, {
			"  VBox", NULL,
			0, 0, "b1", NULL
		}, {
			"   HBox", NULL,
			0, 0, NULL, "h1"
		}, {
			"    +*SpinButton", NULL,
			0, IDC_PRINTER_COPY_PAGENO_SPIN, NULL, NULL
		}, {
			"    +*Label", "  ",
			0, 0, NULL, NULL
		}, {
			" Frame", "Content",
			0, 0, NULL, NULL
		}, {
			"  VBox", NULL,
			0, 0, "b1", NULL
		}, {

#ifdef X88_PRINTER_DRAW_POSTSCRIPT

			"   RadioButton", "PostScript",
			0, IDC_PRINTER_COPY_METAFILE, NULL, "l"

#elif defined(X88_PRINTER_DRAW_CAIRO)

			"   RadioButton", "PDF",
			0, IDC_PRINTER_COPY_METAFILE, NULL, "l"

#endif // X88_PRINTER_DRAW

		}, {
			"   HBox", NULL,
			0, 0, "b1", NULL
		}, {
			"    Label", "  ",
			0, 0, NULL, NULL
		}, {
			"    VBox", NULL,
			0, 0, NULL, NULL
		}, {
			"     HBox", NULL,
			0, 0, NULL, NULL
		}, {
			"      Label", "Zoom Rate",
			0, 0, NULL, NULL
		}, {
			"     HBox", NULL,
			0, 0, NULL, NULL
		}, {
			"      Combo", NULL,
			0, IDC_PRINTER_COPY_ZOOM, NULL, "e0"
		}, {
			"     CheckButton", "Border",
			0, IDC_PRINTER_COPY_BORDER, NULL, NULL
		}, {
			"     CheckButton", "Sprocket Hole",
			0, IDC_PRINTER_COPY_HOLE, NULL, NULL
		}, {
			"   RadioButton", "Text Extraction",
			0, IDC_PRINTER_COPY_TEXT, NULL, "t"
		}
	};
	// printer settings: paper delete dialog
	static SDialogEntry adePrinterPaperDel[] = {
		{
			"Caption", "Printer Settings : Delete Paper",
			0, 0, NULL, NULL
		}, {	// ActionArea
			" Button", "OK",
			0, IDOK, NULL, "fd"
		}, {
			" Button", "Cancel",
			0, IDCANCEL, NULL, "f"
		}, {	// MainVBox
			"+*VBox", NULL,
			0, 0, "b1", NULL
		}, {
			" Frame", "Target",
			0, 0, NULL, NULL
		}, {
			"  VBox", NULL,
			0, 0, NULL, NULL
		}, {
			"   HBox", NULL,
			0, 0, NULL, "h1"
		}, {
			"    +*RadioButton", "Specified Page",
			0, IDC_PRINTER_PAPERDEL_ONEPAGE, NULL, "l"
		}, {
			"    -+*SpinButton", NULL,
			0, IDC_PRINTER_PAPERDEL_PAGENO_SPIN, NULL, NULL
		}, {
			"   HBox", NULL,
			0, 0, NULL, NULL
		}, {
			"    +*RadioButton", "All Pages",
			0, IDC_PRINTER_PAPERDEL_ALLPAGES, NULL, "t"
		}
	};
	// printer settings: paper feed dialog
	static SDialogEntry adePrinterPaperFeed[] = {
		{
			"Caption", "Printer Settings : Feed Paper",
			0, 0, NULL, NULL
		}, {	// ActionArea
			" Button", "OK",
			0, IDOK, NULL, "fd"
		}, {
			" Button", "Cancel",
			0, IDCANCEL, NULL, "f"
		}, {	// MainVBox
			"+*VBox", NULL,
			0, 0, "b1", NULL
		}, {
			" Frame", "Unit",
			0, 0, NULL, NULL
		}, {
			"  VBox", NULL,
			0, 0, NULL, NULL
		}, {
			"   HBox", NULL,
			0, 0, NULL, NULL
		}, {
			"    +*RadioButton", "Page",
			0, IDC_PRINTER_PAPERFEED_PAGE, NULL, "l"
		}, {
			"   HBox", NULL,
			0, 0, NULL, "h1"
		}, {
			"    +*RadioButton", "Line",
			0, IDC_PRINTER_PAPERFEED_LINE, NULL, "t"
		}, {
			"    -+*SpinButton", NULL,
			0, IDC_PRINTER_PAPERFEED_LINE_COUNT_SPIN, NULL, NULL
		}, {
			"   HBox", NULL,
			0, 0, NULL, "h1"
		}, {
			"    +*RadioButton", "Point",
			0, IDC_PRINTER_PAPERFEED_POINT, NULL, "t"
		}, {
			"    -+*SpinButton", NULL,
			0, IDC_PRINTER_PAPERFEED_POINT_COUNT_SPIN, NULL, NULL
		}
	};
	// disk image dialog
	static SDialogEntry adeDiskImage[] = {
		{
			"Caption", "Disk Image",
			0, 0, NULL, NULL
		}, {	// ActionArea
			" Button", "OK",
			0, IDOK, NULL, "fd"
		}, {	// MainVBox
			"+*VBox", NULL,
			0, 0, "b1", NULL
		}, {	// Images
			" +*Frame", "Disk Image File",
			0, 0, NULL, NULL
		}, {
			"  VBox", NULL,
			0, 0, "b1", NULL
		}, {
			"   HBox", NULL,
			0, 0, NULL, NULL
		}, {
			"    Label", "File",
			0, 0, NULL, NULL
		}, {
			"   +*HBox", NULL,
			0, 0, NULL, NULL
		}, {
			"    +*ScrolledWindow", NULL,
			0, 0, "s64,1", "p0,1"
		}, {
			"     List", NULL,
			0, IDC_IMAGES, NULL, "m"
		}, {
			"    VBox", NULL,
			0, 0, NULL, NULL
		}, {
			"     Button", "Add",
			0, IDC_IMAGE_ADD, NULL, NULL
		}, {
			"     Button", "Remove",
			0, IDC_IMAGE_REMOVE, NULL, NULL
		}, {
			"     Button", "Remove All",
			0, IDC_IMAGE_ERASE, NULL, NULL
		}, {	// Drives
			" Frame", "Disk Drive",
			0, 0, NULL, NULL
		}, {
			"   VBox", NULL,
			0, 0, "b1", NULL
		}, {
			"    +*HBox", NULL,
			0, 0, NULL, "h1"
		}, {	// Drive1
			"     +*VBox", NULL,
			0, 0, NULL, NULL
		}, {
			"      HBox", NULL,
			0, 0, NULL, NULL
		}, {
			"       Label", "Drive 1",
			0, 0, NULL, NULL
		}, {
			"       -CheckButton", "P",
			0, IDC_DISK_DRIVE1_PROTECT, NULL, NULL
		}, {
			"      +*Combo", NULL,
			0, IDC_DISK_DRIVE1, NULL, "e0"
		}, {	// Drive2
			"     +*VBox", NULL,
			0, 0, NULL, NULL
		}, {
			"      HBox", NULL,
			1, 0, NULL, NULL
		}, {
			"       Label", "Drive 2",
			1, 0, NULL, NULL
		}, {
			"       -CheckButton", "P",
			1, IDC_DISK_DRIVE2_PROTECT, NULL, NULL
		}, {
			"      +*Combo", NULL,
			1, IDC_DISK_DRIVE2, NULL, "e0"
		}, {
			"    +*HBox", NULL,
			0, 0, NULL, "h1"
		}, {	// Drive3
			"     +*VBox", NULL,
			0, 0, NULL, NULL
		}, {
			"      HBox", NULL,
			2, 0, NULL, NULL
		}, {
			"       Label", "Drive 3",
			2, 0, NULL, NULL
		}, {
			"       -CheckButton", "P",
			2, IDC_DISK_DRIVE3_PROTECT, NULL, NULL
		}, {
			"      +*Combo", NULL,
			2, IDC_DISK_DRIVE3, NULL, "e0"
		}, {	// Drive4
			"     +*VBox", NULL,
			0, 0, NULL, NULL
		}, {
			"      HBox", NULL,
			4, 0, NULL, NULL
		}, {
			"       Label", "Drive 4",
			4, 0, NULL, NULL
		}, {
			"       -CheckButton", "P",
			4, IDC_DISK_DRIVE4_PROTECT, NULL, NULL
		}, {
			"      +*Combo", NULL,
			4, IDC_DISK_DRIVE4, NULL, "e0"
		}
	};
	// tape image dialog
	static SDialogEntry adeTapeImage[] = {
		{
			"Caption", "Tape Image",
			0, 0, NULL, NULL
		}, {	// ActionArea
			" Button", "OK",
			0, IDOK, NULL, "fd"
		}, {	// MainVBox
			"VBox", NULL,
			0, 0, "b1", NULL
		}, {	// Load Image
			" +*Frame", "Image for Loading",
			0, 0, NULL, NULL
		}, {
			"  VBox", NULL,
			0, 0, "b1", NULL
		}, {
			"   +*Frame", NULL,
			0, 0, NULL, NULL
		}, {
			"    Label", "",
			0, IDC_LIMAGE_NAME, NULL, NULL
		}, {
			"   Label", NULL,
			0, 0, "s60,0", NULL
		}, {
			"   +*HBox", NULL,
			0, 0, NULL, NULL
		}, {
			"    +*VBox", NULL,
			0, 0, NULL, NULL
		}, {
			"     +*Label", "",
			0, IDC_LIMAGE_INFO, NULL, NULL
		}, {
			"     +*Label", "",
			0, IDC_LIMAGE_COUNTER, NULL, NULL
		}, {
			"    -VBox", NULL,
			0, 0, NULL, NULL
		}, {
			"     Button", "Open",
			0, IDC_LIMAGE_OPEN, NULL, NULL
		}, {
			"     HBox", NULL,
			0, 0, NULL, NULL
		}, {
			"      +*Button", "<<",
			0, IDC_LIMAGE_REW, NULL, NULL
		}, {
			"      +*Button", ">>",
			0, IDC_LIMAGE_FWD, NULL, NULL
		}, {
			"     Label", NULL,
			0, 0, "s15,0", NULL
		}, {	// Save Image
			" +*Frame", "Image for Saving",
			0, 0, NULL, NULL
		}, {
			"  HBox", NULL,
			0, 0, "b1", NULL
		}, {
			"   +*VBox", NULL,
			0, 0, NULL, NULL
		}, {
			"    +*Label", "",
			0, IDC_SIMAGE_INFO, NULL, NULL
		}, {
			"    +*Label", "",
			0, IDC_SIMAGE_COUNTER, NULL, NULL
		}, {
			"   -VBox", NULL,
			0, 0, NULL, NULL
		}, {
			"    Button", "Save",
			0, IDC_SIMAGE_SAVE, NULL, NULL
		}, {
			"    Button", "Erase",
			0, IDC_SIMAGE_ERASE, NULL, NULL
		}, {
			"    Label", NULL,
			0, 0, "s15,0", NULL
		}
	};
	// RAM writing dialog
	static SDialogEntry adeDebugWriteRam[] = {
		{
			"Caption", "Export RAM",
			0, 0, NULL, NULL
		}, {	// ActionArea
			" Button", "OK",
			0, IDOK, NULL, "fd"
		}, {
			" Button", "Cancel",
			0, IDCANCEL, NULL, "f"
		}, {	// MainVBox
			"VBox", NULL,
			0, 0, "b1", NULL
		}, {
			" Frame", "RAM",
			0, 0, NULL, NULL
		}, {
			"  VBox", NULL,
			0, 0, "b1", NULL
		}, {
			"   CheckButton", "Main RAM0 : 0000H-7FFFH",
			0, IDC_WRITE_MAIN_RAM0, NULL, NULL
		}, {
			"   CheckButton", "Main RAM1 : 8000H-FFFFH",
			0, IDC_WRITE_MAIN_RAM1, NULL, NULL
		}, {
			"   CheckButton", "Fast Text VRAM",
			1, IDC_WRITE_FAST_TVRAM, NULL, NULL
		}, {
			"   CheckButton", "Slow Text VRAM",
			2, IDC_WRITE_SLOW_TVRAM, NULL, NULL
		}, {
			"   CheckButton", "Graphic VRAM0",
			0, IDC_WRITE_GVRAM0, NULL, NULL
		}, {
			"   CheckButton", "Graphic VRAM1",
			0, IDC_WRITE_GVRAM1, NULL, NULL
		}, {
			"   CheckButton", "Graphic VRAM2",
			0, IDC_WRITE_GVRAM2, NULL, NULL
		}, {
			"   CheckButton", "Subsystem RAM",
			0, IDC_WRITE_SUB_RAM, NULL, NULL
		}, {
			"   CheckButton", "Expansion RAM0",
			0, IDC_WRITE_EX_RAM0, NULL, NULL
		}, {
			"   CheckButton", "Expansion RAM1[VAB]",
			0, IDC_WRITE_EX_RAM1, NULL, NULL
		}
	};
	// dis-assemble dialog
	static SDialogEntry adeDisAssemble[] = {
		{
			"Caption", "Disassemble",
			0, 0, NULL, NULL
		}, {	// ActionArea
			" Button", "OK",
			0, IDOK, NULL, "fd"
		}, {	// MainVBox
			"+*VBox", NULL,
			0, 0, "b1", NULL
		}, {
			" Frame", "Range",
			0, 0, NULL, NULL
		}, {
			"  HBox", NULL,
			0, 0, "b1", NULL
		}, {
			"   VBox", NULL,
			0, 0, NULL, NULL
		}, {
			"    HBox", NULL,
			0, 0, NULL, NULL
		}, {
			"     Label", "Address",
			0, 0, NULL, NULL
		}, {
			"    Entry", NULL,
			0, IDC_ADDRESS, NULL, NULL
		}, {
			"   VBox", NULL,
			0, 0, NULL, NULL
		}, {
			"    HBox", NULL,
			0, 0, NULL, NULL
		}, {
			"     Label", "Lines",
			0, 0, NULL, NULL
		}, {
			"    Entry", NULL,
			0, IDC_LINE_COUNT, NULL, NULL
		}, {
			"   VBox", NULL,
			0, 0, NULL, NULL
		}, {
			"    -Button", "Refresh",
			0, IDC_DIS_ASSEMBLE, NULL, NULL
		}, {
			"   Label", NULL,
			0, 0, "s10,1", NULL
		}, {
			" HBox", NULL,
			0, 0, NULL, NULL
		}, {
			"  Label", "Disassembled List",
			0, 0, NULL, NULL
		}, {
			" +*HBox", NULL,
			0, 0, NULL, "s0"
		}, {
			"  +*Text", NULL,
			0, IDC_DIS_ASSEMBLE_LIST, "s40,40", "ve1"
		}, {
			"  VScrollBar", NULL,
			0, 0, NULL, "a"
		}
	};
	// dump dialog
	static SDialogEntry adeDump[] = {
		{
			"Caption", "Dump",
			0, 0, NULL, NULL
		}, {	// ActionArea
			" Button", "OK",
			0, IDOK, NULL, "fd"
		}, {	// MainVBox
			"+*VBox", NULL,
			0, 0, "b1", NULL
		}, {
			" Frame", "Range",
			0, 0, NULL, NULL
		}, {
			"  HBox", NULL,
			0, 0, "b1", NULL
		}, {
			"   VBox", NULL,
			0, 0, NULL, NULL
		}, {
			"    HBox", NULL,
			0, 0, NULL, NULL
		}, {
			"     Label", "Address",
			0, 0, NULL, NULL
		}, {
			"    Entry", NULL,
			0, IDC_ADDRESS, NULL, NULL
		}, {
			"   VBox", NULL,
			0, 0, NULL, NULL
		}, {
			"    HBox", NULL,
			0, 0, NULL, NULL
		}, {
			"     Label", "Byte Count",
			0, 0, NULL, NULL
		}, {
			"    Entry", NULL,
			0, IDC_BYTE_COUNT, NULL, NULL
		}, {
			"   VBox", NULL,
			0, 0, NULL, NULL
		}, {
			"    -Button", "Refresh",
			0, IDC_DUMP, NULL, NULL
		}, {
			"   Label", NULL,
			0, 0, "s10,1", NULL
		}, {
			" HBox", NULL,
			0, 0, NULL, NULL
		}, {
			"  Label", "Dump List",
			0, 0, NULL, NULL
		}, {
			" +*HBox", NULL,
			0, 0, NULL, "s0"
		}, {
			"  +*Text", NULL,
			0, IDC_DUMP_LIST, "s40,40", "ve1"
		}, {
			"  VScrollBar", NULL,
			0, 0, NULL, "a"
		}
	};
	// breakpoint dialog
	static SDialogEntry adeBreakPoint[] = {
		{
			"Caption", "Breakpoint Settings",
			0, 0, NULL, NULL
		}, {	// ActionArea
			" Button", "OK",
			0, IDOK, NULL, "fd"
		}, {	// MainVBox
			"+*VBox", NULL,
			0, 0, "b1", NULL
		}, {
			" Frame", "Position",
			0, 0, NULL, NULL
		}, {
			"  HBox", NULL,
			0, 0, "b1", NULL
		}, {
			"   VBox", NULL,
			0, 0, NULL, NULL
		}, {
			"    HBox", NULL,
			0, 0, NULL, NULL
		}, {
			"     Label", "Address",
			0, 0, NULL, NULL
		}, {
			"    Entry", NULL,
			0, IDC_ADDRESS, NULL, NULL
		}, {
			"   VBox", NULL,
			0, 0, NULL, NULL
		}, {
			"    -Button", "Add",
			0, IDC_ADDRESS_ADD, NULL, NULL
		}, {
			" HBox", NULL,
			0, 0, NULL, NULL
		}, {
			"  Label", "Breakpoint",
			0, 0, NULL, NULL
		}, {
			" +*HBox", NULL,
			0, 0, NULL, NULL
		}, {
			"  +*ScrolledWindow", NULL,
			0, 0, "s1,20", "p0,1"
		}, {
			"   List", NULL,
			0, IDC_BREAK_POINT, NULL, "m"
		}, {
			"  VBox", NULL,
			0, 0, NULL, NULL
		}, {
			"   Button", "Remove",
			0, IDC_BREAK_POINT_REMOVE, NULL, NULL
		}, {
			"   Button", "Remove All",
			0, IDC_BREAK_POINT_ERASE, NULL, NULL
		}
	};
	// version dialog
	static SDialogEntry adeVersion[] = {
		{
			"Caption", "Version Information",
			0, 0, NULL, NULL
		}, {	// ActionArea
			" Button", "OK",
			0, IDOK, NULL, "fd"
		}, {	// MainVBox
			"+*VBox", NULL,
			0, 0, "b1", NULL
		}, {
			" Label", NULL,
			0, 0, "s50,1", NULL
		}, {
			" Label", "X88000 Ver. ",
			0, IDC_VERSION, NULL, NULL
		}, {
			" Label", "by Manuke",
			0, 0, NULL, NULL
		}, {
			" Label", NULL,
			0, 0, "s50,1", NULL
		}
	};

	SDialogEntry* pdeTemplate = NULL;
	nItemCount = 0;

#define SET_DIALOG_ENTRY(ade) \
	pdeTemplate = ade; \
	nItemCount = sizeof(ade)/sizeof(ade[0]);

	switch (nID) {
	case IDD_MESSAGEBOX:
		SET_DIALOG_ENTRY(adeMessageBox);
		break;
	case IDD_ENV_SET:
		SET_DIALOG_ENTRY(adeEnvSet);
		break;
	case IDD_ENV_GENERAL:
		SET_DIALOG_ENTRY(adeEnvGeneral);
		break;
	case IDD_ENV_DISPLAY:
		SET_DIALOG_ENTRY(adeEnvDisplay);
		break;
	case IDD_ENV_INPUT:
		SET_DIALOG_ENTRY(adeEnvInput);
		break;
	case IDD_PRINTER:
		SET_DIALOG_ENTRY(adePrinter);
		break;
	case IDD_PRINTER_COPY:
		SET_DIALOG_ENTRY(adePrinterCopy);
		break;
	case IDD_PRINTER_PAPERDEL:
		SET_DIALOG_ENTRY(adePrinterPaperDel);
		break;
	case IDD_PRINTER_PAPERFEED:
		SET_DIALOG_ENTRY(adePrinterPaperFeed);
		break;
	case IDD_DISK_IMAGE:
		SET_DIALOG_ENTRY(adeDiskImage);
		break;
	case IDD_TAPE_IMAGE:
		SET_DIALOG_ENTRY(adeTapeImage);
		break;
	case IDD_DEBUG_WRITE_RAM:
		SET_DIALOG_ENTRY(adeDebugWriteRam);
		break;
	case IDD_DEBUG_DIS_ASSEMBLE:
		SET_DIALOG_ENTRY(adeDisAssemble);
		break;
	case IDD_DEBUG_DUMP:
		SET_DIALOG_ENTRY(adeDump);
		break;
	case IDD_DEBUG_BREAK_POINT:
		SET_DIALOG_ENTRY(adeBreakPoint);
		break;
	case IDD_VERSION:
		SET_DIALOG_ENTRY(adeVersion);
		break;
	}

#undef SET_DIALOG_ENTRY

	return pdeTemplate;
}

#endif // X88_GUI

////////////////////////////////////////////////////////////
// menu resource

#ifdef X88_GUI_GTK

// get menu entries

GtkItemFactoryEntry* NX88Resource::GetMenuEntries(int& nItemCount) {
	// main menu
	static GtkItemFactoryEntry aifeMenu[] = {
		{
			(char*)"/_System",
			NULL,
			NULL,
			0,
			(char*)"<Branch>"
		}, {
			(char*)"/System/_Reset",
			NULL,
			NULL,
			IDM_RESET,
			NULL
		}, {
			(char*)"/System/---",
			NULL,
			NULL,
			0,
			(char*)"<Separator>"
		}, {
			(char*)"/System/B_ASIC Mode",
			NULL,
			NULL,
			0,
			(char*)"<Branch>"
		}, {
			(char*)"/System/BASIC Mode/_N-BASIC Mode",
			NULL,
			NULL,
			IDM_BASICMODE_N,
			(char*)"<RadioItem>"
		}, {
			(char*)"/System/BASIC Mode/N88-BASIC V_1-S Mode",
			NULL,
			NULL,
			IDM_BASICMODE_V1_S,
			(char*)"<RadioItem2>"
		}, {
			(char*)"/System/BASIC Mode/N88-BASIC V1-_H Mode",
			NULL,
			NULL,
			IDM_BASICMODE_V1_H,
			(char*)"<RadioItem2>"
		}, {
			(char*)"/System/BASIC Mode/N88-BASIC V_2 Mode",
			NULL,
			NULL,
			IDM_BASICMODE_V2,
			(char*)"<RadioItem2>"
		}, {
			(char*)"/System/BASIC Mode/---",
			NULL,
			NULL,
			0,
			(char*)"<Separator>"
		}, {
			(char*)"/System/BASIC Mode/N_80-BASIC V1 Mode",
			NULL,
			NULL,
			IDM_BASICMODE_N80_V1,
			(char*)"<RadioItem2>"
		}, {
			(char*)"/System/BASIC Mode/N8_0-BASIC V2 Mode",
			NULL,
			NULL,
			IDM_BASICMODE_N80_V2,
			(char*)"<RadioItem2>"
		}, {
			(char*)"/System/_Clock",
			NULL,
			NULL,
			0,
			(char*)"<Branch>"
		}, {
			(char*)"/System/Clock/_4MHz Mode",
			NULL,
			NULL,
			IDM_BASE_CLOCK_4,
			(char*)"<RadioItem>"
		}, {
			(char*)"/System/Clock/_8MHz Mode",
			NULL,
			NULL,
			IDM_BASE_CLOCK_8,
			(char*)"<RadioItem2>"
		}, {
			(char*)"/System/_Boost Mode",
			(char*)"<Alt>B",
			NULL,
			IDM_BOOST_MODE,
			(char*)"<CheckItem>"
		/* unsupport
		}, {
			(char*)"/System/_Fullscreen Mode",
			(char*)"<Alt>Return",
			NULL,
			IDM_FULL_SCREEN_MODE,
			(char*)"<CheckItem>"
		*/
		}, {
			(char*)"/System/---",
			NULL,
			NULL,
			0,
			(char*)"<Separator>"
		}, {
			(char*)"/System/C_lipboard",
			NULL,
			NULL,
			0,
			(char*)"<Branch>"
		}, {
			(char*)"/System/Clipboard/Export Screenshot : XWD(_C)",
			NULL,
			NULL,
			IDM_CLIPBOARD_COPY_BITMAP,
			NULL
		}, {
			(char*)"/System/Clipboard/Copy : _Text",
			NULL,
			NULL,
			IDM_CLIPBOARD_COPY_TEXT,
			NULL
		}, {
			(char*)"/System/Clipboard/_Paste : Text",
			NULL,
			NULL,
			IDM_CLIPBOARD_PASTE_TEXT,
			NULL
		}, {
			(char*)"/System/---",
			NULL,
			NULL,
			0,
			(char*)"<Separator>"
		}, {
			(char*)"/System/_Environment Settings...",
			NULL,
			NULL,
			IDM_ENV_SET,
			NULL
		}, {
			(char*)"/System/_Parallel Port",
			NULL,
			NULL,
			0,
			(char*)"<Branch>"
		}, {
			(char*)"/System/Parallel Port/_Device Settings...",
			NULL,
			NULL,
			IDM_PARALLEL_DEVICE_SETTING,
			NULL
		}, {
			(char*)"/System/Parallel Port/---",
			NULL,
			NULL,
			0,
			(char*)"<Separator>"
		}, {
			(char*)"/System/Parallel Port/%s(_0)",
			NULL,
			NULL,
			IDM_PARALLEL_DEVICE0,
			(char*)"<RadioItem>"
		}, {
			(char*)"/System/Parallel Port/%s(_1)",
			NULL,
			NULL,
			IDM_PARALLEL_DEVICE1,
			(char*)"<RadioItem2>"
		}, {
			(char*)"/System/Parallel Port/%s(_2)",
			NULL,
			NULL,
			IDM_PARALLEL_DEVICE2,
			(char*)"<RadioItem2>"
		}, {
			(char*)"/System/Parallel Port/%s(_3)",
			NULL,
			NULL,
			IDM_PARALLEL_DEVICE3,
			(char*)"<RadioItem2>"
		}, {
			(char*)"/System/Parallel Port/%s(_4)",
			NULL,
			NULL,
			IDM_PARALLEL_DEVICE4,
			(char*)"<RadioItem2>"
		}, {
			(char*)"/System/Parallel Port/%s(_5)",
			NULL,
			NULL,
			IDM_PARALLEL_DEVICE5,
			(char*)"<RadioItem2>"
		}, {
			(char*)"/System/Parallel Port/%s(_6)",
			NULL,
			NULL,
			IDM_PARALLEL_DEVICE6,
			(char*)"<RadioItem2>"
		}, {
			(char*)"/System/Parallel Port/%s(_7)",
			NULL,
			NULL,
			IDM_PARALLEL_DEVICE7,
			(char*)"<RadioItem2>"
		}, {
			(char*)"/System/Parallel Port/%s(_8)",
			NULL,
			NULL,
			IDM_PARALLEL_DEVICE8,
			(char*)"<RadioItem2>"
		}, {
			(char*)"/System/Parallel Port/%s(_9)",
			NULL,
			NULL,
			IDM_PARALLEL_DEVICE9,
			(char*)"<RadioItem2>"
		}, {
			(char*)"/System/---",
			NULL,
			NULL,
			0,
			(char*)"<Separator>"
		}, {
			(char*)"/System/E_xit X88000",
			NULL,
			NULL,
			IDM_EXIT,
			NULL
		}, {
			(char*)"/_Image",
			NULL,
			NULL,
			0,
			(char*)"<Branch>"
		}, {
			(char*)"/Image/_Disk Image...",
			(char*)"<Alt>D",
			NULL,
			IDM_DISK_IMAGE,
			NULL
		}, {
			(char*)"/Image/_Tape Image...",
			(char*)"<Alt>T",
			NULL,
			IDM_TAPE_IMAGE,
			NULL
		}, {
			(char*)"/Image/_Memory Image...",
			(char*)"<Alt>M",
			NULL,
			IDM_MEMORY_IMAGE,
			NULL
		}, {
			(char*)"/Debu_g",
			NULL,
			NULL,
			0,
			(char*)"<Branch>"
		}, {
			(char*)"/Debug/Main CPU Debug(_1)",
			NULL,
			NULL,
			IDM_DEBUG_MAIN,
			(char*)"<CheckItem>"
		}, {
			(char*)"/Debug/Sub CPU Debug(_2)",
			NULL,
			NULL,
			IDM_DEBUG_SUB,
			(char*)"<CheckItem>"
		}, {
			(char*)"/Debug/---",
			NULL,
			NULL,
			0,
			(char*)"<Separator>"
		}, {
			(char*)"/Debug/Execute Debu_g",
			(char*)"<Alt>F5",
			NULL,
			IDM_DEBUG_EXECUTE,
			(char*)"<CheckItem>"
		}, {
			(char*)"/Debug/---",
			NULL,
			NULL,
			0,
			(char*)"<Separator>"
		}, {
			(char*)"/Debug/Execute _Step",
			(char*)"<Alt>F10",
			NULL,
			IDM_DEBUG_EXECUTE_STEP,
			NULL
		}, {
			(char*)"/Debug/Execute _Trace",
			(char*)"<Alt>F11",
			NULL,
			IDM_DEBUG_EXECUTE_TRACE,
			NULL
		}, {
			(char*)"/Debug/Execute Ste_p2",
			(char*)"<Alt>F12",
			NULL,
			IDM_DEBUG_EXECUTE_STEP2,
			NULL
		}, {
			(char*)"/Debug/---",
			NULL,
			NULL,
			0,
			(char*)"<Separator>"
		}, {
			(char*)"/Debug/Record Execution L_og",
			NULL,
			NULL,
			IDM_DEBUG_LOG,
			(char*)"<CheckItem>"
		}, {
			(char*)"/Debug/Export RAM(_W)...",
			NULL,
			NULL,
			IDM_DEBUG_WRITE_RAM,
			NULL
		}, {
			(char*)"/Debug/---",
			NULL,
			NULL,
			0,
			(char*)"<Separator>"
		}, {
			(char*)"/Debug/Disassemb_le...",
			NULL,
			NULL,
			IDM_DEBUG_DIS_ASSEMBLE,
			NULL
		}, {
			(char*)"/Debug/_Dump...",
			NULL,
			NULL,
			IDM_DEBUG_DUMP,
			NULL
		}, {
			(char*)"/Debug/_Breakpoint Settings...",
			NULL,
			NULL,
			IDM_DEBUG_BREAK_POINT,
			NULL
		}, {
			(char*)"/_Help",
			NULL,
			NULL,
			0,
			(char*)"<Branch>"
		}, {
			(char*)"/Help/_About...",
			NULL,
			NULL,
			IDM_VERSION,
			NULL
		}
	};

	nItemCount = sizeof(aifeMenu)/sizeof(aifeMenu[0]);
	return aifeMenu;
}

#endif // X88_GUI
